@@ -1,5 +1,5 @@
 // DireWeightContainer.h is a part of the PYTHIA event generator.
-// Copyright (C) 2023 Stefan Prestel, Torbjorn Sjostrand.
+// Copyright (C) 2025 Stefan Prestel, Torbjorn Sjostrand.
 // PYTHIA is licenced under the GNU GPL v2 or later, see COPYING for details.
 // Please respect the MCnet Guidelines, see GUIDELINES for details.
 
@@ -101,7 +101,7 @@ public:
   }
   void setup();
 
-  void initPtrs(BeamParticle* beamAIn, BeamParticle* beamBIn,
+  void initPtrs(BeamParticlePtr beamAIn, BeamParticlePtr beamBIn,
     Settings* settingsPtrIn, Info* infoPtrIn, DireInfo* direInfoPtrIn) {
     beamA    = beamAIn;
     beamB    = beamBIn;
@@ -219,8 +219,8 @@ private:
   unordered_map<string,double> enhanceFactors;
   map<ulong, double> trialEnhancements;
 
-  BeamParticle* beamA;
-  BeamParticle* beamB;
+  BeamParticlePtr beamA;
+  BeamParticlePtr beamB;
   Info* infoPtr;
   DireInfo* direInfoPtr;
 
