@@ -1,13 +1,11 @@
 // HIBasics.h is a part of the PYTHIA event generator.
-// Copyright (C) 2023 Torbjorn Sjostrand.
+// Copyright (C) 2025 Torbjorn Sjostrand.
 // PYTHIA is licenced under the GNU GPL v2 or later, see COPYING for details.
 // Please respect the MCnet Guidelines, see GUIDELINES for details.
 
-// This file contains the definition of the EventInfo and
-// HIUserHooks classes, as well as the HIUnits namespace.
+// This file contains the definition of the EventInfo class.
 //
 // EventInfo: stores full nucleon-nucleon events with corresponding Info.
-// HIUserHooks: User hooks for HeavyIons models.
 
 #ifndef Pythia8_HIBasics_H
 #define Pythia8_HIBasics_H
@@ -22,26 +20,6 @@ class SubCollision;
 
 //==========================================================================
 
-// The HIUnits namespace defines the unitsystem used by the heavy ion
-// machinery in Pythia8. In particular all lengths are in femtometer
-// and cross sections are therefore in squared femtometer.
-namespace HIUnits {
-
-// Lengths
-const double femtometer = 1.0;
-const double millimeter = 1.0e12;
-
-// Cross sections
-const double femtometer2 = 1.0;
-const double millibarn = 0.1;
-const double nanobarn = 1.0e-7;
-
-}
-
-using namespace HIUnits;
-
-//==========================================================================
-
 // Class for storing Events and Info objects.
 
 class EventInfo {
