@@ -1,5 +1,5 @@
 // MergingHooks.h is a part of the PYTHIA event generator.
-// Copyright (C) 2021 Torbjorn Sjostrand.
+// Copyright (C) 2025 Torbjorn Sjostrand.
 // PYTHIA is licenced under the GNU GPL v2 or later, see COPYING for details.
 // Please respect the MCnet Guidelines, see GUIDELINES for details.
 
@@ -173,7 +173,8 @@ public:
     doMGMergingSave(false),
     doKTMergingSave(false),
     doPTLundMergingSave(false),
-    doCutBasedMergingSave(false), includeMassiveSave(),
+    doCutBasedMergingSave(false),
+    doDynamicMergingSave(false), includeMassiveSave(),
     enforceStrongOrderingSave(),
     orderInRapiditySave(), pickByFullPSave(), pickByPoPT2Save(),
     includeRedundantSave(), pickBySumPTSave(), allowColourShufflingSave(),
@@ -192,9 +193,12 @@ public:
     doUNLOPSSubtNLOSave(false),
     doUMEPSTreeSave(false),
     doUMEPSSubtSave(false),
-    doEstimateXSection(false), applyVeto(),
+    doEstimateXSection(false),
+    doRuntimeAMCATNLOInterfaceSave(false),
+    applyVeto(),
     doRemoveDecayProducts(false), muMISave(), kFactor0jSave(), kFactor1jSave(),
-    kFactor2jSave(), tmsValueSave(), tmsValueNow(), DparameterSave(),
+    kFactor2jSave(), tmsValueSave(), tmsValueNow(),
+    DparameterSave(), SparameterSave(),
     nJetMaxSave(), nJetMaxNLOSave(),
     doOrderHistoriesSave(true),
     doCutOnRecStateSave(false),
@@ -283,8 +287,15 @@ public:
     else return tmsValueNow;
   }
   double tmsCut() {
-    if(doCutBasedMergingSave) return 0.;
-    else return tmsValueSave;
+    if (doCutBasedMergingSave) return 0.;
+    if (doDynamicMergingSave) {
+      // Fetch fixed merging cut and factorisation scale.
+      const double QbarCut = tmsValueSave;
+      const double Q2      = infoPtr->Q2DIS();
+      // Calculate dynamic merging scale, cf. eq. (2) in arXiv:0912.3715.
+      return QbarCut/sqrt(1. + pow2(QbarCut/SparameterSave)/Q2);
+    }
+    return tmsValueSave;
   }
   void tms( double tmsIn ) { tmsValueNow = tmsIn; }
 
@@ -336,8 +347,11 @@ public:
   bool doPTLundMerging() { return doPTLundMergingSave;}
   // Function to determine if cut based merging should be applied.
   bool doCutBasedMerging() { return doCutBasedMergingSave;}
+  // Function to determine if merging with dynamic scale should be applied.
+  bool doDynamicMerging() { return doDynamicMergingSave;}
   bool doCKKWLMerging() { return (doUserMergingSave || doMGMergingSave
-    || doKTMergingSave || doPTLundMergingSave || doCutBasedMergingSave); }
+      || doKTMergingSave || doPTLundMergingSave || doCutBasedMergingSave
+      || doDynamicMergingSave); }
   // Functions to determine if and which part of  UMEPS merging
   // should be applied
   bool doUMEPSTree() { return doUMEPSTreeSave;}
@@ -358,6 +372,10 @@ public:
   bool doUNLOPSSubtNLO() { return doUNLOPSSubtNLOSave;}
   bool doUNLOPSMerging() { return (doUNLOPSTreeSave || doUNLOPSLoopSave
                              || doUNLOPSSubtSave || doUNLOPSSubtNLOSave); }
+
+  // Function to determine if we have a runtime interface to aMC@NLO.
+  bool doRuntimeAMCATNLOInterface() { return doRuntimeAMCATNLOInterfaceSave;}
+
   // Return the number clustering steps that have actually been done.
   int nRecluster() { return nReclusterSave;}
 
@@ -381,13 +399,11 @@ public:
   bool allowEffectiveVertex( vector<int> in, vector<int> out) {
     if ( getProcessString().compare("ta+ta->jj") == 0
       || getProcessString().compare("ta-ta+>jj") == 0 ) {
-      int nInFermions(0), nOutFermions(0), nOutBosons(0);
+      int nInFermions(0), nOutFermions(0);
       for (int i=0; i < int(in.size()); ++i)
         if (abs(in[i])<20) nInFermions++;
-      for (int i=0; i < int(out.size()); ++i) {
+      for (int i=0; i < int(out.size()); ++i)
         if (abs(out[i])<20) nOutFermions++;
-        if (abs(out[i])>20) nOutBosons++;
-      }
       return (nInFermions%2==0 && nOutFermions%2==0);
     }
     return false;
@@ -447,13 +463,11 @@ public:
 
   // Flag to indicate trial shower usage.
   void doIgnoreEmissions( bool doIgnoreIn ) {
-    doIgnoreEmissionsSave = doIgnoreIn;
-  }
+    doIgnoreEmissionsSave = doIgnoreIn;}
   // Function to allow not counting a trial emission.
   virtual bool canVetoEmission() { return !doIgnoreEmissionsSave; }
   // Function to check if emission should be rejected.
   virtual bool doVetoEmission( const Event& );
-  virtual bool usesVincia() {return false;}
 
   //----------------------------------------------------------------------//
   // Functions used as clusterings / probabilities
@@ -461,6 +475,7 @@ public:
 
   bool useShowerPluginSave;
   virtual bool useShowerPlugin() { return useShowerPluginSave; }
+  virtual bool usesVincia() {return false;}
 
   //----------------------------------------------------------------------//
   // Functions to retrieve if merging weight should countin the internal
@@ -509,7 +524,7 @@ public:
 
   // Flags for merging procedure definition.
   bool   doUserMergingSave, doMGMergingSave, doKTMergingSave,
-         doPTLundMergingSave, doCutBasedMergingSave,
+         doPTLundMergingSave, doCutBasedMergingSave, doDynamicMergingSave,
          includeMassiveSave, enforceStrongOrderingSave, orderInRapiditySave,
          pickByFullPSave, pickByPoPT2Save, includeRedundantSave,
          pickBySumPTSave, allowColourShufflingSave, resetHardQRenSave,
@@ -520,7 +535,7 @@ public:
 
   double scaleSeparationFactorSave, nonJoinedNormSave,
          fsrInRecNormSave, herwigAcollFSRSave, herwigAcollISRSave,
-         pT0ISRSave, pTcutSave;
+         pT0ISRSave, pTcutSave, pTminISRSave, pTminFSRSave;
   bool   doNL3TreeSave, doNL3LoopSave, doNL3SubtSave;
   bool   doUNLOPSTreeSave, doUNLOPSLoopSave, doUNLOPSSubtSave,
          doUNLOPSSubtNLOSave;
@@ -529,7 +544,13 @@ public:
   // Flag to only do phase space cut, rejecting events below the tms cut.
   bool   doEstimateXSection;
 
-  bool applyVeto;
+  // Flag for runtime aMC@NLO interface. Needed for aMC@NLO-Delta.
+  bool   doRuntimeAMCATNLOInterfaceSave;
+
+  // Flag for postponing event vetos. If false, events are not vetoed
+  // based on the merging scale in CKKW-L merging, but have to be
+  // vetoed by the user in the main program.
+  bool   applyVeto;
 
   // Save input event in case decay products need to be detached.
   Event inputEvent;
@@ -545,7 +566,7 @@ public:
   double kFactor2jSave;
 
   // Saved members.
-  double tmsValueSave, tmsValueNow, DparameterSave;
+  double tmsValueSave, tmsValueNow, DparameterSave, SparameterSave;
   int nJetMaxSave;
   int nJetMaxNLOSave;
 
