@@ -1,5 +1,5 @@
 // SigmaProcess.h is a part of the PYTHIA event generator.
-// Copyright (C) 2022 Torbjorn Sjostrand.
+// Copyright (C) 2025 Torbjorn Sjostrand.
 // PYTHIA is licenced under the GNU GPL v2 or later, see COPYING for details.
 // Please respect the MCnet Guidelines, see GUIDELINES for details.
 
@@ -91,7 +91,7 @@ public:
   virtual ~SigmaProcess() {}
 
   // Perform simple initialization and store pointers.
-  void init(BeamParticle* beamAPtrIn, BeamParticle* beamBPtrIn,
+  void init(BeamParticlePtr beamAPtrIn, BeamParticlePtr beamBPtrIn,
     SLHAinterface* slhaInterfacePtrIn = 0);
 
   // Store or replace Les Houches pointer.
