@@ -1,5 +1,5 @@
 // SimpleTimeShower.h is a part of the PYTHIA event generator.
-// Copyright (C) 2022 Torbjorn Sjostrand.
+// Copyright (C) 2025 Torbjorn Sjostrand.
 // PYTHIA is licenced under the GNU GPL v2 or later, see COPYING for details.
 // Please respect the MCnet Guidelines, see GUIDELINES for details.
 
@@ -15,6 +15,9 @@
 
 namespace Pythia8 {
 
+// Forward declaration of SplitOnia;
+class SplitOnia;
+
 //==========================================================================
 
 // Data on radiating dipole ends; only used inside SimpleTimeShower class.
@@ -26,44 +29,49 @@ public:
   // Constructors.
   TimeDipoleEnd() : iRadiator(-1), iRecoiler(-1), pTmax(0.), colType(0),
     chgType(0), gamType(0), weakType(0), isrType(0), system(0), systemRec(0),
-    MEtype(0), iMEpartner(-1), weakPol(0), isOctetOnium(false),
-    isHiddenValley(false), colvType(0), MEmix(0.), MEorder(true),
-    MEsplit(true), MEgluinoRec(false), isFlexible(false), hasJunction(false),
-    flavour(), iAunt(), mRad(), m2Rad(), mRec(), m2Rec(), mDip(), m2Dip(),
-    m2DipCorr(), pT2(), m2(), z(), mFlavour(), asymPol(), flexFactor(),
-    pAccept() { }
+    MEtype(0), iMEpartner(-1), weakPol(0), oniumType(0), isHiddenValley(false),
+    colvType(0), MEmix(0.), MEorder(true), MEsplit(true), MEgluinoRec(false),
+    isFlexible(false), hasJunction(false), flavour(), iAunt(),
+    mRad(), m2Rad(), mRec(), m2Rec(), mDip(), m2Dip(), m2DipCorr(), pT2(),
+    m2(), z(), mFlavour(), asymPol(), flexFactor(), pAccept() {}
   TimeDipoleEnd(int iRadiatorIn, int iRecoilerIn, double pTmaxIn = 0.,
     int colIn = 0, int chgIn = 0, int gamIn = 0, int weakTypeIn = 0,
     int isrIn = 0, int systemIn = 0, int MEtypeIn = 0, int iMEpartnerIn = -1,
-    int weakPolIn = 0, bool isOctetOniumIn = false,
+    int weakPolIn = 0, int oniumTypeIn = 0,
     bool isHiddenValleyIn = false, int colvTypeIn = 0, double MEmixIn = 0.,
     bool MEorderIn = true, bool MEsplitIn = true, bool MEgluinoRecIn = false,
     bool isFlexibleIn = false, bool hasJunctionIn = false) :
     iRadiator(iRadiatorIn), iRecoiler(iRecoilerIn), pTmax(pTmaxIn),
     colType(colIn), chgType(chgIn), gamType(gamIn), weakType(weakTypeIn),
     isrType(isrIn), system(systemIn), systemRec(systemIn), MEtype(MEtypeIn),
-    iMEpartner(iMEpartnerIn), weakPol(weakPolIn), isOctetOnium(isOctetOniumIn),
+    iMEpartner(iMEpartnerIn), weakPol(weakPolIn), oniumType(oniumTypeIn),
     isHiddenValley(isHiddenValleyIn), colvType(colvTypeIn), MEmix(MEmixIn),
     MEorder (MEorderIn), MEsplit(MEsplitIn), MEgluinoRec(MEgluinoRecIn),
-    isFlexible(isFlexibleIn), hasJunction(hasJunctionIn), flavour(), iAunt(),
-    mRad(), m2Rad(), mRec(), m2Rec(), mDip(), m2Dip(), m2DipCorr(), pT2(),
-    m2(), z(), mFlavour(), asymPol(), flexFactor(), pAccept()  { }
+    isFlexible(isFlexibleIn), hasJunction(hasJunctionIn),
+    flavour(), iAunt(), mRad(), m2Rad(), mRec(), m2Rec(), mDip(), m2Dip(),
+    m2DipCorr(), pT2(), m2(), z(), mFlavour(), asymPol(), flexFactor(),
+    pAccept() {}
 
   // Basic properties related to dipole and matrix element corrections.
   int    iRadiator, iRecoiler;
   double pTmax;
   int    colType, chgType, gamType, weakType, isrType, system, systemRec,
-         MEtype, iMEpartner, weakPol;
-  bool   isOctetOnium, isHiddenValley;
+         MEtype, iMEpartner, weakPol, oniumType;
+  bool   isHiddenValley;
   int    colvType;
   double MEmix;
   bool   MEorder, MEsplit, MEgluinoRec, isFlexible;
   bool   hasJunction;
 
+
   // Properties specific to current trial emission.
   int    flavour, iAunt;
   double mRad, m2Rad, mRec, m2Rec, mDip, m2Dip, m2DipCorr,
          pT2, m2, z, mFlavour, asymPol, flexFactor, pAccept;
+  double m2A{0}, m2B{0}, m2C{0}, m2gg{-1};
+
+  // Pointer to an onium emission object, if present.
+  shared_ptr<SplitOnia> emissionPtr{};
 
 };
 
@@ -79,37 +87,42 @@ public:
   SimpleTimeShower() : hasWeaklyRadiated(), iSysSel(), pTmaxFudge(),
     pTLastBranch(), doQCDshower(), doQEDshowerByQ(), doQEDshowerByL(),
     doQEDshowerByOther(), doQEDshowerByGamma(), doWeakShower(),
-    doMEcorrections(), doMEextended(), doMEafterFirst(), doPhiPolAsym(),
-    doPhiPolAsymHard(), doInterleave(), doInterleaveResDec(),
-    allowBeamRecoil(), dampenBeamRecoil(), recoilToColoured(),
-    useFixedFacScale(), allowRescatter(), canVetoEmission(), doHVshower(),
-    brokenHVsym(), globalRecoil(), useLocalRecoilNow(), doSecondHard(),
-    hasUserHooks(), singleWeakEmission(), alphaSuseCMW(), vetoWeakJets(),
-    allowMPIdipole(), weakExternal(), recoilDeadCone(), doDipoleRecoil(),
-    doPartonVertex(), pTmaxMatch(), pTdampMatch(), alphaSorder(),
-    alphaSnfmax(), nGluonToQuark(), weightGluonToQuark(), alphaEMorder(),
-    nGammaToQuark(), nGammaToLepton(), nCHV(), idHV(), alphaHVorder(),
-    nMaxGlobalRecoil(), weakMode(), pTdampFudge(), mc(), mb(), m2c(), m2b(),
-    renormMultFac(), factorMultFac(), fixedFacScale2(), alphaSvalue(),
-    alphaS2pi(), Lambda3flav(), Lambda4flav(), Lambda5flav(), Lambda3flav2(),
-    Lambda4flav2(), Lambda5flav2(), scaleGluonToQuark(), extraGluonToQuark(),
-    pTcolCutMin(), pTcolCut(), pT2colCut(), pTchgQCut(), pT2chgQCut(),
-    pTchgLCut(), pT2chgLCut(), pTweakCut(), pT2weakCut(), mMaxGamma(),
-    m2MaxGamma(), octetOniumFraction(), octetOniumColFac(), mZ(), gammaZ(),
-    thetaWRat(), mW(), gammaW(), CFHV(), nFlavHV(), alphaHVfix(), LambdaHV(),
-    pThvCut(), pT2hvCut(), mHV(), pTmaxFudgeMPI(), weakEnhancement(),
-    vetoWeakDeltaR2(), twoHard(), dopTlimit1(), dopTlimit2(), dopTdamp(),
-    pT2damp(), kRad(), kEmt(), pdfScale2(), doTrialNow(), canEnhanceEmission(),
+    doMEcorrections(), doMEextended(), doMEafterFirst(),
+    doPhiPolAsym(), doPhiPolAsymHard(), doInterleave(),
+    doInterleaveResDec(), allowBeamRecoil(), dampenBeamRecoil(),
+    useFixedFacScale(), allowRescatter(), canVetoEmission(),
+    doHVshower(), brokenHVsym(), setLambdaHV(), globalRecoil(),
+    useLocalRecoilNow(), doSecondHard(), hasUserHooks(),
+    singleWeakEmission(), alphaSuseCMW(), vetoWeakJets(),
+    allowMPIdipole(), weakExternal(), recoilDeadCone(),
+    doDipoleRecoil(), doPartonVertex(), recoilRFUseParents(false),
+    pTmaxMatch(), pTdampMatch(), alphaSorder(), alphaSnfmax(),
+    nGluonToQuark(), weightGluonToQuark(), recoilStrategyRF(),
+    alphaEMorder(), nGammaToQuark(), nGammaToLepton(), nCHV(),
+    nFlavHV(), idHV(), alphaHVorder(), nMaxGlobalRecoil(), weakMode(),
+    pTdampFudge(), mc(), mb(), m2c(), m2b(), renormMultFac(),
+    factorMultFac(), fixedFacScale2(), alphaSvalue(), alphaS2pi(),
+    Lambda3flav(), Lambda4flav(), Lambda5flav(), Lambda3flav2(),
+    Lambda4flav2(), Lambda5flav2(), scaleGluonToQuark(),
+    extraGluonToQuark(), weightRF(1), pTcolCutMin(), pTcolCut(),
+    pT2colCut(), pTchgQCut(), pT2chgQCut(), pTchgLCut(), pT2chgLCut(),
+    pTweakCut(), pT2weakCut(), mMaxGamma(), m2MaxGamma(), mZ(),
+    gammaZ(), thetaWRat(), mW(), gammaW(), CFHV(), alphaHVfix(),
+    alphaHVref(), LambdaHV(), pThvCut(), pT2hvCut(), mHV(),
+    pTmaxFudgeMPI(), weakEnhancement(), vetoWeakDeltaR2(), twoHard(),
+    dopTlimit1(), dopTlimit2(), dopTdamp(), pT2damp(), kRad(), kEmt(),
+    pdfScale2(), doTrialNow(), canEnhanceEmission(),
     canEnhanceTrial(), canEnhanceET(), doUncertaintiesNow(), dipSel(),
     iDipSel(), nHard(), nFinalBorn(), nMaxGlobalBranch(), nGlobal(),
-    globalRecoilMode(), limitMUQ(), weakHardSize() { beamOffset = 0;}
+    globalRecoilMode(), limitMUQ(), weakHardSize() { beamOffset = 0;
+    pdfMode = 0; useSystems = true; }
 
   // Destructor.
   virtual ~SimpleTimeShower() override {}
 
   // Initialize alphaStrong and related pTmin parameters.
-  virtual void init( BeamParticle* beamAPtrIn = 0,
-    BeamParticle* beamBPtrIn = 0) override;
+  virtual void init( BeamParticlePtr beamAPtrIn = nullptr,
+    BeamParticlePtr beamBPtrIn = nullptr) override;
 
   // Find whether to limit maximum scale of emissions, and whether to dampen.
   virtual bool limitPTmax( Event& event, double Q2Fac = 0.,
@@ -120,7 +133,8 @@ public:
     int nBranchMax = 0) override;
 
   // Top-level routine for QED radiation in hadronic decay to two leptons.
-  virtual int showerQED( int i1, int i2, Event& event, double pTmax) override;
+  virtual int showerQED( int i1, int i2, Event& event, double pTmax = -1.)
+    override;
 
   // Prepare process-level event for shower + interleaved resonance decays.
   // Usage: prepareProcess( process, event, iPos).
@@ -180,12 +194,24 @@ public:
   // Provide the pT scale of the last branching in the above shower.
   virtual double pTLastInShower() override {return pTLastBranch;}
 
+  // Functions to directly extract the probability of no emission between two
+  // scales. These functions are not used in the Pythia core code, but can be
+  // used by external programs to interface with the shower directly.
+  double noEmissionProbability( double pTbegAll, double pTendAll, double m2dip,
+    int id, int type, double s = -1., double x = -1.) override;
+  double pTnext( vector<TimeDipoleEnd> dipEnds, Event event, double pTbegAll,
+    double pTendAll, double m2dip, int id, int type, double s = -1.,
+    double x = -1.);
+  int pdfMode;
+  bool useSystems;
+
 private:
 
   // Constants: could only be changed in the code itself.
   static const double MCMIN, MBMIN, SIMPLIFYROOT, XMARGIN, XMARGINCOMB,
-         TINYPDF, LARGEM2, THRESHM2, LAMBDA3MARGIN, WEAKPSWEIGHT, WG2QEXTRA,
-         REJECTFACTOR, PROBLIMIT;
+         WTRATIOMAX,TINYPDF, LARGEM2, THRESHM2, LAMBDA3MARGIN1ORD,
+         LAMBDA3MARGIN2ORD,WEAKPSWEIGHT, WG2QEXTRA, REJECTFACTOR,
+         PROBLIMIT;
   static const int NLOOPMAX;
   // Rescatter: try to fix up recoil between systems
   static const bool   FIXRESCATTER, VETONEGENERGY;
@@ -201,27 +227,30 @@ private:
          doQEDshowerByGamma, doWeakShower, doMEcorrections, doMEextended,
          doMEafterFirst, doPhiPolAsym, doPhiPolAsymHard, doInterleave,
          doInterleaveResDec, allowBeamRecoil, dampenBeamRecoil,
-         recoilToColoured, useFixedFacScale, allowRescatter, canVetoEmission,
-         doHVshower, brokenHVsym, globalRecoil, useLocalRecoilNow,
+         useFixedFacScale, allowRescatter, canVetoEmission, doHVshower,
+         brokenHVsym, setLambdaHV, globalRecoil, useLocalRecoilNow,
          doSecondHard, hasUserHooks, singleWeakEmission, alphaSuseCMW,
          vetoWeakJets, allowMPIdipole, weakExternal, recoilDeadCone,
-         doDipoleRecoil, doPartonVertex;
+         doDipoleRecoil, doPartonVertex, recoilRFUseParents;
+  int    pdfModeSave;
   int    pTmaxMatch, pTdampMatch, alphaSorder, alphaSnfmax, nGluonToQuark,
-         weightGluonToQuark, alphaEMorder, nGammaToQuark, nGammaToLepton,
-         nCHV, idHV, alphaHVorder, nMaxGlobalRecoil, weakMode;
+         weightGluonToQuark, recoilStrategyRF, alphaEMorder, nGammaToQuark,
+         nGammaToLepton, nCHV, nFlavHV, idHV, alphaHVorder, nMaxGlobalRecoil,
+         weakMode;
   double pTdampFudge, mc, mb, m2c, m2b, renormMultFac, factorMultFac,
          fixedFacScale2, alphaSvalue, alphaS2pi, Lambda3flav, Lambda4flav,
          Lambda5flav, Lambda3flav2, Lambda4flav2, Lambda5flav2,
-         scaleGluonToQuark, extraGluonToQuark, pTcolCutMin, pTcolCut,
-         pT2colCut, pTchgQCut, pT2chgQCut, pTchgLCut, pT2chgLCut,
-         pTweakCut, pT2weakCut, mMaxGamma, m2MaxGamma, octetOniumFraction,
-         octetOniumColFac, mZ, gammaZ, thetaWRat, mW, gammaW, CFHV, nFlavHV,
-         alphaHVfix, LambdaHV, pThvCut, pT2hvCut, mHV, pTmaxFudgeMPI,
-         weakEnhancement, vetoWeakDeltaR2;
+         scaleGluonToQuark, extraGluonToQuark, weightRF,
+         pTcolCutMin, pTcolCut, pT2colCut, pTchgQCut, pT2chgQCut,
+         pTchgLCut, pT2chgLCut, pTweakCut, pT2weakCut, mMaxGamma, m2MaxGamma,
+         mZ, gammaZ, thetaWRat, mW, gammaW, CFHV,
+         alphaHVfix, alphaHVref, LambdaHV, pThvCut, pT2hvCut, mHV,
+         pTmaxFudgeMPI, weakEnhancement, vetoWeakDeltaR2;
 
-  // alphaStrong and alphaEM calculations.
+  // alphaStrong, alphaEM and alpha_HV calculations.
   AlphaStrong alphaS;
   AlphaEM     alphaEM;
+  AlphaSUN    alphaHV;
 
   // Weak matrix elements used for corrections both of ISR and FSR.
   SimpleWeakShowerMEs  simpleWeakShowerMEs;
@@ -252,7 +281,15 @@ private:
     bool limitPTmaxIn = true);
   // Special setup for weak dipoles if already specified in info ptr.
   void setupWeakdipExternal(Event& event, bool limitPTmaxIn = true);
-  void setupHVdip( int iSys, int i, Event& event, bool limitPTmaxIn = true);
+  void setupHVdip( int iSys, int i, int colvType, Event& event,
+    bool limitPTmaxIn = true);
+
+  // Apply ME corrections for a specific subsystem.
+  bool applyMECorrections(const Event& event, TimeDipoleEnd* dipBranch,
+    int iSysBranch);
+
+  // Special setup for onium.
+  void regenerateOniumDipoles(Event & event);
 
   // Evolve a QCD dipole end.
   void pT2nextQCD( double pT2begDip, double pT2sel, TimeDipoleEnd& dip,
@@ -270,6 +307,10 @@ private:
   void pT2nextHV( double pT2begDip, double pT2sel, TimeDipoleEnd& dip,
     Event& );
 
+  // Evolve an onium dipole end.
+  void pT2nextOnium( double pT2begDip, double pT2sel, TimeDipoleEnd& dip,
+    Event& );
+
   // Find kind of QCD ME correction.
   void findMEtype( Event& event, TimeDipoleEnd& dip);
 
@@ -324,12 +365,21 @@ private:
   vector<int> weak2to2lines;
   int weakHardSize;
 
-  // Settings and member variables for interleaved resonance decays
+  // Settings and member variables for interleaved resonance decays.
   bool doFSRinResonances{};
   int resDecScaleChoice{-1}, iHardResDecSav{}, nRecurseResDec{};
   vector<int> idResDecSav;
   vector<double> pTresDecSav;
 
+  // Settings and containers for control of MECs.
+  bool skipFirstMECinHardProc;
+  vector<int> skipFirstMECinResDecIDs{};
+
+  // Onium emissions.
+  bool doOniumShower{false};
+  vector<SplitOniaPtr> oniumEmissions;
+  set<double> oniumThresholds;
+
 };
 
 //==========================================================================
